@@ -16,10 +16,21 @@
 #define nmhlibdir(file) NMHLIBDIR#file
 
 
+/*
+ * Find the location of a format or configuration
+ * file, and return its absolute pathname.
+ *
+ * 1) If already absolute pathname, then leave unchanged.
+ * 2) Next, if it begins with ~user, then expand it.
+ * 3) Next, check in nmh Mail directory.
+ * 4) Next, check in nmh `etc' directory.
+ *
+ */
+
 char *
 etcpath (char *file)
 {
-    static char epath[BUFSIZ];
+    static char epath[PATH_MAX];
     char *cp;
 #ifdef MHRC
     char *pp;
@@ -32,15 +43,17 @@ etcpath (char *file)
 
     switch (*file) {
 	case '/': 
+	    /* If already absolute pathname, return it */
 	    return file;
 
 #ifdef MHRC
 	case '~': 
+	    /* Expand ~username */
 	    if ((cp = strchr(pp = file + 1, '/')))
 		*cp++ = '\0';
-	    if (*pp == '\0')
+	    if (*pp == '\0') {
 		pp = mypath;
-	    else
+	    } else {
 		if ((pw = getpwnam (pp)))
 		    pp = pw->pw_dir;
 		else {
@@ -48,21 +61,25 @@ etcpath (char *file)
 			*--cp = '/';
 		    goto try_it;
 		}
+	    }
 
-	    sprintf (epath, "%s/%s", pp, cp ? cp : "");
+	    snprintf (epath, sizeof(epath), "%s/%s", pp, cp ? cp : "");
 	    if (cp)
 		*--cp = '/';
 
 	    if (access (epath, R_OK) != NOTOK)
 		return epath;	/* else fall */
-    try_it: ;
+try_it:
 #endif /* MHRC */
 
 	default: 
+	    /* Check nmh Mail directory */
 	    if (access ((cp = m_mailpath (file)), R_OK) != NOTOK)
 		return cp;
     }
-    sprintf (epath, nmhetcdir(/%s), file);
+
+    /* Check nmh `etc' directory */
+    snprintf (epath, sizeof(epath), nmhetcdir(/%s), file);
     return (access (epath, R_OK) != NOTOK ? epath : file);
 }
 
@@ -114,6 +131,14 @@ char *usequence = "Unseen-Sequence";
 char *psequence = "Previous-Sequence";
 char *nsequence = "Sequence-Negation";
 
+/* profile entries for storage locations */
+char *nmhstorage = "nmh-storage";
+char *nmhcache = "nmh-cache";
+char *nmhprivcache = "nmh-private-cache";
+
+/* profile entry for external ftp access command */
+char *nmhaccessftp = "nmh-access-ftp";
+
 char *mhlibdir = NMHLIBDIR;
 char *mhetcdir = NMHETCDIR;
 
@@ -274,7 +299,7 @@ char *sendproc = nmhbindir (/send);
  * to display non-text (MIME) messages.
  */
 
-char *showmimeproc = nmhbindir (/mhn);
+char *showmimeproc = nmhbindir (/mhshow);
 
 /*
  * This is the default program called by "show" to filter
@@ -310,7 +335,7 @@ char *whomproc = nmhbindir (/whom);
  * editor, such as vi or emacs, but any editor will work.
  */
 
-char *sysed = DEFAULT_EDITOR;
+char *defaulteditor = DEFAULT_EDITOR;
 
 /* 
  * This is the global nmh alias file.  It is somewhat obsolete, since
