@@ -43,7 +43,7 @@ void context_read (void);
 void context_replace (char *, char *);
 void context_save (void);
 char *copy (char *, char *);
-char **copyip (char **, char **);
+char **copyip (char **, char **, int);
 void cpydata (int, int, char *, char *);
 void cpydgst (int, int, char *, char *);
 int decode_rfc2047 (char *, char *);
@@ -59,12 +59,13 @@ struct msgs *folder_realloc (struct msgs *, int, int);
 int gans (char *, struct swit *);
 char **getans (char *, struct swit *);
 int getanswer (char *);
+char **getarguments (char *, int, char **, int);
 char *getcpy (char *);
 char *getfolder(int);
 int lkclose(int, char*);
 int lkfclose(FILE *, char *);
 FILE *lkfopen(char *, char *);
-int lkopen(char *, int);
+int lkopen(char *, int, mode_t);
 int m_atoi (char *);
 char *m_backup (char *);
 int m_convert (struct msgs *, char *);
@@ -76,7 +77,6 @@ char *m_maildir (char *);
 char *m_mailpath (char *);
 char *m_name (int);
 int m_putenv (char *, char *);
-void m_readefs (struct node **, FILE *, char *, int);
 char *m_scratch (char *, char *);
 char *m_tmpfil (char *);
 void m_unknown(FILE *);
@@ -92,6 +92,7 @@ void print_version (char *);
 void push (void);
 char *pwd (void);
 char *r1bindex(char *, int);
+void readconfig (struct node **, FILE *, char *, int);
 int refile (char **, char *);
 int remdir (char *);
 int seq_addmsg (struct msgs *, char *, int, int, int);
@@ -121,7 +122,7 @@ int vfgets (FILE *, char **);
 char *write_charset_8bit (void);
 
 #ifdef RPATHS
-int get_returnpath (char *, char *);
+int get_returnpath (char *, int, char *, int);
 #endif
 
 /*
