@@ -33,17 +33,17 @@ refile (char **arg, char *file)
     fflush(stdout);
 
     switch (pid = vfork()) {
-	case NOTOK: 
+	case -1: 
 	    advise ("fork", "unable to");
-	    return NOTOK;
+	    return -1;
 
-	case OK: 
+	case 0: 
 	    execvp (fileproc, vec);
 	    fprintf (stderr, "unable to exec ");
 	    perror (fileproc);
 	    _exit (-1);
 
 	default: 
-	    return (pidwait (pid, NOTOK));
+	    return (pidwait (pid, -1));
     }
 }
