@@ -47,8 +47,8 @@ main (int argc, char **argv)
 {
     int inplace = 0, quietsw = 0, verbosw = 0;
     int msgp = 0, hi, msgnum, numburst;
-    char *cp, *maildir, *folder = NULL, buf[100], **ap;
-    char **argp, *arguments[MAXARGS], *msgs[MAXARGS];
+    char *cp, *maildir, *folder = NULL, buf[BUFSIZ];
+    char **argp, **arguments, *msgs[MAXARGS];
     struct smsg *smsgs;
     struct msgs *mp;
 
@@ -56,13 +56,11 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -75,7 +73,8 @@ main (int argc, char **argv)
 		adios (NULL, "-%s unknown\n", cp);
 
 	    case HELPSW: 
-		sprintf (buf, "%s [+folder] [msgs] [switches]", invo_name);
+		snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
+			invo_name);
 		print_help (buf, switches, 1);
 		done (1);
 	    case VERSIONSW:
@@ -129,7 +128,8 @@ main (int argc, char **argv)
     if (!(mp = folder_read (folder)))
 	adios (NULL, "unable to read folder %s", folder);
 
-    if (mp->hghmsg == 0)
+    /* check for empty folder */
+    if (mp->nummsg == 0)
 	adios (NULL, "no messages in %s", folder);
 
     /* parse all the message ranges/sequences and set SELECTED */
@@ -293,8 +293,8 @@ burst (struct msgs **mpp, int msgnum, struct smsg *smsgs, int numburst,
      */
     if (inplace) {
 	for (i = mp->hghmsg; j > msgnum; i--, j--) {
-	    strcpy (f1, m_name (i));
-	    strcpy (f2, m_name (j));
+	    strncpy (f1, m_name (i), sizeof(f1));
+	    strncpy (f2, m_name (j), sizeof(f2));
 	    if (does_exist (mp, j)) {
 		if (verbosw)
 		    printf ("message %d becomes message %d\n", j, i);
@@ -313,8 +313,8 @@ burst (struct msgs **mpp, int msgnum, struct smsg *smsgs, int numburst,
     /* new hghmsg is hghmsg + numburst */
     i = inplace ? msgnum + numburst : mp->hghmsg;
     for (j = numburst; j >= (inplace ? 0 : 1); i--, j--) {
-	strcpy (f1, m_name (i));
-	strcpy (f2, m_scratch ("", invo_name));
+	strncpy (f1, m_name (i), sizeof(f1));
+	strncpy (f2, m_scratch ("", invo_name), sizeof(f2));
 	if (verbosw && i != msgnum)
 	    printf ("message %d of digest %d becomes message %d\n", j, msgnum, i);
 
@@ -327,7 +327,7 @@ burst (struct msgs **mpp, int msgnum, struct smsg *smsgs, int numburst,
 	fclose (out);
 
 	if (i == msgnum) {
-	    strcpy (f3, m_backup (f1));
+	    strncpy (f3, m_backup (f1), sizeof(f3));
 	    if (rename (f1, f3) == NOTOK)
 		admonish (f3, "unable to rename %s to", f1);
 	}
