@@ -37,9 +37,8 @@ int
 main (int argc, char **argv)
 {
     int i, autof = 0;
-    char *cp, *path;
-    char buf[100], *dp;
-    char *arguments[MAXARGS], **argp;
+    char *cp, *path, buf[BUFSIZ];
+    char *dp, **arguments, **argp;
     struct node *np;
     struct passwd *pw;
     struct stat st;
@@ -49,7 +48,7 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    copyip (argv + 1, arguments);
+    arguments = getarguments (invo_name, argc, argv, 0);
     argp = arguments;
 
     while ((dp = *argp++)) {
@@ -62,7 +61,7 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown\n", dp);
 
 		case HELPSW:
-		    sprintf (buf, "%s [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches]", invo_name);
 		    print_help (buf, switches, 0);
 		    done (1);
 		case VERSIONSW:
@@ -171,7 +170,7 @@ query:
      * nmh `etc' directory, then read it also.
      */
     if ((in = fopen (mh_defaults, "r"))) {
-	m_readefs (&np->n_next, in, mh_defaults, 0);
+	readconfig (&np->n_next, in, mh_defaults, 0);
 	fclose (in);
     }
 
