@@ -228,6 +228,8 @@ static char *ovtxt;
 static char *onelp;
 
 static char *parptr;
+
+static int num_ignores = 0;
 static char *ignores[MAXARGS];
 
 static  jmp_buf env;
@@ -291,29 +293,25 @@ int
 mhl (int argc, char **argv)
 {
     int length = 0, nomore = 0;
-    int width = 0, vecp = 0, i;
-    char *cp, *folder = NULL;
-    char *form = NULL, **ap, **argp;
-    char buf[80], *arguments[MAXARGS], *files[MAXARGS];
+    int i, width = 0, vecp = 0;
+    char *cp, *folder = NULL, *form = NULL;
+    char buf[BUFSIZ], *files[MAXARGS];
+    char **argp, **arguments;
 
     invo_name = r1bindex (argv[0], '/');
 
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
+    argp = arguments;
+
     if ((cp = getenv ("MHLDEBUG")) && *cp)
 	mhldebug++;
 
-    if ((cp = context_find (invo_name)) != NULL) {
-	ap = brkstring (getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
-    argp = arguments;
-
     if ((cp = getenv ("FACEPROC")))
 	faceproc = cp;
 
-    vecp = 0;
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, mhlswitches)) {
@@ -324,7 +322,7 @@ mhl (int argc, char **argv)
 		    adios (NULL, "-%s unknown\n", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [switches] [files ...]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches] [files ...]", invo_name);
 		    print_help (buf, mhlswitches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -460,10 +458,11 @@ mhl (int argc, char **argv)
 	if (digest) {
 	    printf ("%s", delim4);
 	    if (volume == 0) {
-	       sprintf (buf, "End of %s Digest\n", digest);
-	     }
-	    else
-	      sprintf (buf, "End of %s Digest [Volume %d Issue %d]\n", digest, volume, issue);
+		snprintf (buf, sizeof(buf), "End of %s Digest\n", digest);
+	    } else {
+		snprintf (buf, sizeof(buf), "End of %s Digest [Volume %d Issue %d]\n",
+			digest, volume, issue);
+	    }
 	    i = strlen (buf);
 	    for (cp = buf + i; i > 1; i--)
 		*cp++ = '*';
@@ -543,13 +542,30 @@ mhl_format (char *file, int length, int width)
 	}
 
 	parptr = bp;
-	strcpy (name, parse());
+	strncpy (name, parse(), sizeof(name));
 	switch (*parptr) {
 	    case '\0': 
 	    case ',': 
 	    case '=': 
+		/*
+		 * Split this list of fields to ignore, and copy
+		 * it to the end of the current "ignores" list.
+		 */
 		if (!strcasecmp (name, "ignores")) {
-		    ip = copyip (brkstring (getcpy (++parptr), ",", NULL), ip);
+		    char **tmparray, **p;
+		    int n = 0;
+
+		    /* split the fields */
+		    tmparray = brkstring (getcpy (++parptr), ",", NULL);
+
+		    /* count number of fields split */
+		    p = tmparray;
+		    while (*p++)
+			n++;
+
+		    /* copy pointers to split fields to ignores array */
+		    ip = copyip (tmparray, ip, MAXARGS - num_ignores);
+		    num_ignores += n;
 		    continue;
 		}
 		parptr = bp;
@@ -596,7 +612,7 @@ mhl_format (char *file, int length, int width)
 		    c1->c_offset, c1->c_ovoff, c1->c_width,
 		    c1->c_cwidth, c1->c_length);
 	    fprintf (stderr, "\tflags=%s\n",
-		    snprintb (buffer, BUFSIZ, (unsigned) c1->c_flags, LBITS));
+		    snprintb (buffer, sizeof(buffer), (unsigned) c1->c_flags, LBITS));
 	}
     }
 
@@ -636,7 +652,7 @@ evalvar (struct mcomp *c1)
 
     if (!*parptr)
 	return 0;
-    strcpy (name, parse ());
+    strncpy (name, parse(), sizeof(name));
 
     if (!strcasecmp (name, "component")) {
 	if (ptos (name, &c1->c_text))
@@ -744,7 +760,7 @@ parse (void)
     char *cp;
     static char result[NAMESZ];
 
-    for (cp = result; *parptr; parptr++) {
+    for (cp = result; *parptr && (cp - result < NAMESZ); parptr++) {
 	c = *parptr;
 	if (isalnum (c)
 		|| c == '.'
@@ -838,7 +854,7 @@ mhlfile (FILE *fp, char *mname, int ofilen, int ofilec)
 		break;
 
 	    case ISTTY: 
-		strcpy (buf, "\n");
+		strncpy (buf, "\n", sizeof(buf));
 		if (ofilec > 1) {
 		    if (SOprintf ("Press <return> to list \"%s\"...", mname)) {
 			if (ofilen > 1)
@@ -929,7 +945,7 @@ mhlfile (FILE *fp, char *mname, int ofilen, int ofilec)
 		    if (dobody && !strcasecmp (c1->c_name, "body")) {
 			if ((holder.c_text = malloc (sizeof(buf))) == NULL)
 			    adios (NULL, "unable to allocate buffer memory");
-			strcpy (holder.c_text, buf);
+			strncpy (holder.c_text, buf, sizeof(buf));
 			while (state == BODY) {
 			    putcomp (c1, &holder, BODYCOMP);
 			    state = m_getfld (state, name, holder.c_text,
@@ -1589,7 +1605,7 @@ doface (struct mcomp *c1)
 	    vec[vecp++] = "-e";
 	    if (sleepsw != NOTOK) {
 		vec[vecp++] = "-s";
-		sprintf (buffer, "%d", sleepsw);
+		snprintf (buffer, sizeof(buffer), "%d", sleepsw);
 		vec[vecp++] = buffer;
 	    }
 	    vec[vecp] = NULL;
