@@ -60,7 +60,7 @@ static int eom_action (int);
 static FILE *mhl_action (char *);
 static int ask (int);
 static int is_nontext (int);
-static int getws (char *, char *, int, struct Msg *);
+static int get_fields (char *, char *, int, struct Msg *);
 static int msgsort (struct Msg *, struct Msg *);
 static int subsort (struct Msg *, struct Msg *);
 static char *sosmash (char *, char *);
@@ -76,7 +76,7 @@ forkcmd (char **args, char *pgm)
     char *vec[MAXARGS];
 
     vec[0] = r1bindex (pgm, '/');
-    copyip (args, vec + 1);
+    copyip (args, vec + 1, MAXARGS - 1);
 
     if (fmsh) {
 	context_del (pfolder);
@@ -164,7 +164,7 @@ distcmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case DIHELP: 
-		    sprintf (buf, "%s [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [msgs] [switches]", cmd_name);
 		    print_help (buf, distswit, 1);
 		    return;
 
@@ -266,7 +266,7 @@ explcmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case EXHELP: 
-		    sprintf (buf, "%s [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [msgs] [switches]", cmd_name);
 		    print_help (buf, explswit, 1);
 		    return;
 
@@ -495,7 +495,7 @@ filecmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case FIHELP: 
-		    sprintf (buf, "%s +folder... [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s +folder... [msgs] [switches]", cmd_name);
 		    print_help (buf, fileswit, 1);
 		    return;
 
@@ -657,7 +657,7 @@ foldcmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case FLHELP: 
-		    sprintf (buf, "%s [+folder] [msg] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [+folder] [msg] [switches]", cmd_name);
 		    print_help (buf, foldswit, 1);
 		    return;
 
@@ -720,7 +720,7 @@ foldcmd (char **args)
 	    }
 	}
 	else {
-	    strcpy (buf, folder);
+	    strncpy (buf, folder, sizeof(buf));
 	    if (expand (buf) == NOTOK)
 		return;
 	    folder = buf;
@@ -795,10 +795,12 @@ fast: ;
 		DMAXFOLDER, "", DMAXFOLDER - 2, "", DMAXFOLDER - 2, "",
 		DMAXFOLDER - 2, "");
 	printf (args ? "%22s  " : "%s ", fmsh ? fmsh : mp->foldpath);
-	if (mp->hghmsg == 0)
+
+	/* check for empty folder */
+	if (mp->nummsg == 0) {
 	    printf ("has   no messages%*s",
 		    mp->msgflags & OTHERS ? DMAXFOLDER * 2 + 4 : 0, "");
-	else {
+	} else {
 	    printf ("has %*d message%s (%*d-%*d)",
 		    DMAXFOLDER, mp->nummsg, mp->nummsg != 1 ? "s" : "",
 		    DMAXFOLDER, mp->lowmsg, DMAXFOLDER, mp->hghmsg);
@@ -874,7 +876,7 @@ forwcmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case FOHELP: 
-		    sprintf (buf, "%s [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [msgs] [switches]", cmd_name);
 		    print_help (buf, forwswit, 1);
 		    return;
 
@@ -929,7 +931,7 @@ forwcmd (char **args)
     }
 
 					/* foil search of .mh_profile */
-    sprintf (buf, "%sXXXXXX", invo_name);
+    snprintf (buf, sizeof(buf), "%sXXXXXX", invo_name);
     vec[0] = (char *)mktemp (buf);
     vec[vecp++] = "-file";
     vec[vecp] = NULL;
@@ -941,7 +943,7 @@ forwcmd (char **args)
     seq_setprev (mp);
 
     if (filter) {
-	strcpy (buf, filter);
+	strncpy (buf, filter, sizeof(buf));
 	if (expand (buf) == NOTOK)
 	    return;
 	if (access (filter = getcpy (etcpath (buf)), R_OK) == NOTOK) {
@@ -964,7 +966,7 @@ forw (char *proc, char *filter, int vecp, char **vec)
     char tmpfil[80], *args[MAXARGS];
     FILE *out;
 
-    strcpy (tmpfil, m_tmpfil (invo_name));
+    strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
     interrupted = 0;
     if (filter)
 	switch (child_id = fork ()) {
@@ -1132,7 +1134,7 @@ markcmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case MHELP: 
-		    sprintf (buf, "%s [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [msgs] [switches]", cmd_name);
 		    print_help (buf, markswit, 1);
 		    return;
 
@@ -1210,10 +1212,10 @@ markcmd (char **args)
 	printf ("invo_name=%s mypath=%s defpath=%s\n",
 		invo_name, mypath, defpath);
 	printf ("ctxpath=%s context flags=%s\n",
-		ctxpath, snprintb (buf, BUFSIZ, (unsigned) ctxflags, DBITS));
+		ctxpath, snprintb (buf, sizeof(buf), (unsigned) ctxflags, DBITS));
 	printf ("foldpath=%s flags=%s\n",
 		mp->foldpath,
-		snprintb (buf, BUFSIZ, (unsigned) mp->msgflags, FBITS));
+		snprintb (buf, sizeof(buf), (unsigned) mp->msgflags, FBITS));
 	printf ("hghmsg=%d lowmsg=%d nummsg=%d curmsg=%d\n",
 		mp->hghmsg, mp->lowmsg, mp->nummsg, mp->curmsg);
 	printf ("lowsel=%d hghsel=%d numsel=%d\n",
@@ -1258,10 +1260,14 @@ markcmd (char **args)
 		    msgnum++)
 		if (is_selected (mp, msgnum)) {
 		    printf ("%*d: id=%d top=%d start=%ld stop=%ld %s\n",
-			    DMAXFOLDER, msgnum,
-			    Msgs[msgnum].m_bboard_id, Msgs[msgnum].m_top,
-			    Msgs[msgnum].m_start, Msgs[msgnum].m_stop,
-			    snprintb (buf, BUFSIZ, (unsigned) mp->msgstats[msgnum - mp->lowoff],
+			DMAXFOLDER,
+			msgnum,
+			Msgs[msgnum].m_bboard_id,
+			Msgs[msgnum].m_top,
+			(long) Msgs[msgnum].m_start,
+			(long) Msgs[msgnum].m_stop,
+			snprintb (buf, sizeof(buf),
+				(unsigned) mp->msgstats[msgnum - mp->lowoff],
 				seq_bits (mp)));
 		    if (Msgs[msgnum].m_scanl)
 			printf ("%s", Msgs[msgnum].m_scanl);
@@ -1355,7 +1361,7 @@ mhncmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case MHNHELPSW:
-		    sprintf (buf, "%s [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [msgs] [switches]", cmd_name);
 		    print_help (buf, mhnswit, 1);
 		    return;
 
@@ -1464,7 +1470,7 @@ packcmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case PAHELP: 
-		    sprintf (buf, "%s [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [msgs] [switches]", cmd_name);
 		    print_help (buf, packswit, 1);
 		    return;
 
@@ -1641,7 +1647,7 @@ pickcmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case PIHELP: 
-		    sprintf (buf, "%s [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [msgs] [switches]", cmd_name);
 		    print_help (buf, pickswit, 1);
 		    return;
 
@@ -1827,7 +1833,7 @@ replcmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case REHELP: 
-		    sprintf (buf, "%s [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [msgs] [switches]", cmd_name);
 		    print_help (buf, replswit, 1);
 		    return;
 
@@ -1917,7 +1923,7 @@ rmmcmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case RMHELP: 
-		    sprintf (buf, "%s [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [msgs] [switches]", cmd_name);
 		    print_help (buf, rmmswit, 1);
 		    return;
 	    }
@@ -1966,7 +1972,7 @@ rmm (void)
 	else
 	    for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++)
 		if (is_selected (mp, msgnum)) {
-		    strcpy (buffer, m_backup (cp = m_name (msgnum)));
+		    strncpy (buffer, m_backup (cp = m_name (msgnum)), sizeof(buffer));
 		    if (rename (cp, buffer) == NOTOK)
 			admonish (buffer, "unable to rename %s to", cp);
 		}
@@ -2058,7 +2064,7 @@ scancmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case SCHELP: 
-		    sprintf (buf, "%s [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [msgs] [switches]", cmd_name);
 		    print_help (buf, scanswit, 1);
 		    return;
 
@@ -2304,7 +2310,7 @@ showcmd (char **args)
 		    vec[vecp++] = --cp;
 		    continue;
 		case SHHELP: 
-		    sprintf (buf, "%s %s[switches] [switches for showproc]",
+		    snprintf (buf, sizeof(buf), "%s %s[switches] [switches for showproc]",
 			    cmd_name, mode ? NULL : "[msgs] ");
 		    print_help (buf, showswit, 1);
 		    return;
@@ -2520,7 +2526,7 @@ ask (int msgnum)
 }
 
 
-#include <h/mhn.h>
+#include <h/mime.h>
 
 static int
 is_nontext (int msgnum)
@@ -2735,7 +2741,7 @@ sortcmd (char **args)
 		    fprintf (stderr, "-%s unknown\n", cp);
 		    return;
 		case SOHELP: 
-		    sprintf (buf, "%s [msgs] [switches]", cmd_name);
+		    snprintf (buf, sizeof(buf), "%s [msgs] [switches]", cmd_name);
 		    print_help (buf, sortswit, 1);
 		    return;
 
@@ -2799,7 +2805,7 @@ sortcmd (char **args)
 	    Msgs[msgnum].m_scanl = NULL;
 	}
 	if (is_selected (mp, msgnum)) {
-	    if (getws (datesw, subjsw, msgnum, &Msgs[msgnum]))
+	    if (get_fields (datesw, subjsw, msgnum, &Msgs[msgnum]))
 		twscopy (&Msgs[msgnum].m_tb,
 			msgnum != mp->lowsel ? &Msgs[msgnum - 1].m_tb : &tb);
 	}
@@ -2829,12 +2835,12 @@ sortcmd (char **args)
 
 
 /* 
- * getws - parse message, and get date and subject if needed.  We'll use
- * the msgp->m_tb tws struct for the date, and overload the msgp->m_scanl
- * field with our subject string.
+ * get_fields - parse message, and get date and subject if needed.
+ * We'll use the msgp->m_tb tws struct for the date, and overload
+ * the msgp->m_scanl field with our subject string.
  */
 static int
-getws (char *datesw, char *subjsw, int msgnum, struct Msg *msgp)
+get_fields (char *datesw, char *subjsw, int msgnum, struct Msg *msgp)
 {
     int	state, gotdate = 0;
     char *bp, buf[BUFSIZ], name[NAMESZ];
@@ -2976,7 +2982,7 @@ process (int msgnum, char *proc, int vecp, char **vec)
     FILE *out;
 
     if (fmsh) {
-	strcpy (tmpfil, m_name (msgnum));
+	strncpy (tmpfil, m_name (msgnum), sizeof(tmpfil));
 	context_del (pfolder);
 	context_replace (pfolder, fmsh);/* update current folder   */
 	seq_save (mp);
@@ -2984,21 +2990,21 @@ process (int msgnum, char *proc, int vecp, char **vec)
 	goto ready;
     }
 
-    strcpy (tmpfil, m_scratch ("", invo_name));
+    strncpy (tmpfil, m_scratch ("", invo_name), sizeof(tmpfil));
     if ((out = fopen (tmpfil, "w")) == NULL) {
-	int     olderr;
-	extern int  errno;
-	char    newfil[80];
+	int olderr;
+	extern int errno;
+	char newfil[80];
 
 	olderr = errno;
-	strcpy (newfil, m_tmpfil (invo_name));
+	strncpy (newfil, m_tmpfil (invo_name), sizeof(newfil));
 	if ((out = fopen (newfil, "w")) == NULL) {
 	    errno = olderr;
 	    advise (tmpfil, "unable to create temporary file");
 	    return NOTOK;
+	} else {
+	    strncpy (tmpfil, newfil, sizeof(tmpfil));
 	}
-	else
-	    strcpy (tmpfil, newfil);
     }
     copy_message (msgnum, out);
     fclose (out);
