@@ -136,8 +136,8 @@ main (int argc, char **argv)
     int nedit = 0, nwhat = 0;
     char *cp, *cwd, *dp, *maildir, *file = NULL;
     char *folder = NULL, *msg = NULL, *dfolder = NULL;
-    char *dmsg = NULL, *ed = NULL, drft[BUFSIZ], buf[100];
-    char **ap, **argp, *arguments[MAXARGS];
+    char *dmsg = NULL, *ed = NULL, drft[BUFSIZ], buf[BUFSIZ];
+    char **argp, **arguments;
     struct msgs *mp = NULL;
     struct stat st;
     FILE *in;
@@ -150,13 +150,11 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+
+    /* read user profile/context */
+    context_read();
+
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     while ((cp = *argp++)) {
@@ -169,7 +167,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s: [+folder] [msg] [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s: [+folder] [msg] [switches]",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (0);
 		case VERSIONSW:
@@ -334,13 +333,13 @@ main (int argc, char **argv)
 try_it_again:
 
 #ifdef MHE
-    strcpy (drft, buildsw ? m_maildir ("reply")
-			  : m_draft (dfolder, NULL, NOUSE, &isdf));
+    strncpy (drft, buildsw ? m_maildir ("reply")
+			  : m_draft (dfolder, NULL, NOUSE, &isdf), sizeof(drft));
 
     /* Check if a draft exists */
     if (!buildsw && stat (drft, &st) != NOTOK) {
 #else
-    strcpy (drft, m_draft (dfolder, dmsg, NOUSE, &isdf));
+    strncpy (drft, m_draft (dfolder, dmsg, NOUSE, &isdf), sizeof(drft));
 
     /* Check if a draft exists */
     if (stat (drft, &st) != NOTOK) {
@@ -393,7 +392,8 @@ try_it_again:
 	if (!(mp = folder_read (folder)))
 	    adios (NULL, "unable to read folder %s", folder);
 
-	if (mp->hghmsg == 0)
+	/* check for empty folder */
+	if (mp->nummsg == 0)
 	    adios (NULL, "no messages in %s", folder);
 
 	/* parse the message range/sequence/name and set SELECTED */
