@@ -11,6 +11,13 @@
 #include <zotnet/tws/tws.h>
 #include <errno.h>
 
+/*
+ * We allocate space for message names (msgs array)
+ * this number of elements at a time.
+ */
+#define MAXMSGS  256
+
+
 static struct swit switches[] = {
 #define	CLRSW	0
     { "clear", 0 },
@@ -58,12 +65,12 @@ int
 main (int argc, char **argv)
 {
     int clearflag = 0, hdrflag = 0, ontty;
-    int width = 0, msgp = 0, revflag = 0;
-    int i, state, msgnum;
+    int width = 0, revflag = 0;
+    int i, state, msgnum, nummsgs, maxmsgs;
     int seqnum[NUMATTRS], unseen, num_unseen_seq = 0;
     char *cp, *maildir, *file = NULL, *folder = NULL;
-    char *form = NULL, *format = NULL, buf[100], **ap;
-    char **argp, *nfs, *arguments[MAXARGS], *msgs[MAXARGS];
+    char *form = NULL, *format = NULL, buf[BUFSIZ];
+    char **argp, *nfs, **arguments, **msgs;
     struct msgs *mp;
     FILE *in;
 
@@ -71,16 +78,26 @@ main (int argc, char **argv)
     setlocale(LC_ALL, "");
 #endif
     invo_name = r1bindex (argv[0], '/');
+
+    /* read user profile/context */
+    context_read();
+
     mts_init (invo_name);
-    if ((cp = context_find (invo_name))) {
-	ap = brkstring (cp = getcpy (cp), " ", "\n");
-	ap = copyip (ap, arguments);
-    } else {
-	ap = arguments;
-    }
-    copyip (argv + 1, ap);
+    arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    /*
+     * Allocate the initial space to record message
+     * names, ranges, and sequences.
+     */
+    nummsgs = 0;
+    maxmsgs = MAXMSGS;
+    if (!(msgs = (char **) malloc ((size_t) (maxmsgs * sizeof(*msgs)))))
+	adios (NULL, "unable to allocate storage");
+
+    /*
+     * Parse arguments
+     */
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -91,7 +108,8 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    sprintf (buf, "%s [+folder] [msgs] [switches]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
+			invo_name);
 		    print_help (buf, switches, 1);
 		    done (1);
 		case VERSIONSW:
@@ -149,7 +167,17 @@ main (int argc, char **argv)
 	    else
 		folder = path (cp + 1, *cp == '+' ? TFOLDER : TSUBCWF);
 	} else {
-	    msgs[msgp++] = cp;
+	    /*
+	     * Check if we need to allocate more space
+	     * for message names/ranges/sequences.
+	     */
+	    if (nummsgs >= maxmsgs) {
+		maxmsgs += MAXMSGS;
+		if (!(msgs = (char **) realloc (msgs,
+			(size_t) (maxmsgs * sizeof(*msgs)))))
+		    adios (NULL, "unable to reallocate msgs storage");
+	    }
+	    msgs[nummsgs++] = cp;
 	}
     }
 
@@ -165,7 +193,7 @@ main (int argc, char **argv)
      * We are scanning a maildrop file
      */
     if (file) {
-	if (msgp)
+	if (nummsgs)
 	    adios (NULL, "\"msgs\" not allowed with -file");
 	if (folder)
 	    adios (NULL, "\"+folder\" not allowed with -file");
@@ -200,8 +228,8 @@ main (int argc, char **argv)
      * We are scanning a folder
      */
 
-    if (!msgp)
-	msgs[msgp++] = "all";
+    if (!nummsgs)
+	msgs[nummsgs++] = "all";
     if (!folder)
 	folder = getfolder (1);
     maildir = m_maildir (folder);
@@ -213,11 +241,12 @@ main (int argc, char **argv)
     if (!(mp = folder_read (folder)))
 	adios (NULL, "unable to read folder %s", folder);
 
-    if (mp->hghmsg == 0)
+    /* check for empty folder */
+    if (mp->nummsg == 0)
 	adios (NULL, "no messages in %s", folder);
 
     /* parse all the message ranges/sequences and set SELECTED */
-    for (msgnum = 0; msgnum < msgp; msgnum++)
+    for (msgnum = 0; msgnum < nummsgs; msgnum++)
 	if (!m_convert (mp, msgs[msgnum]))
 	    done(1);
     seq_setprev (mp);			/* set the Previous-Sequence */
@@ -230,8 +259,8 @@ main (int argc, char **argv)
      * Get the sequence number for each sequence
      * specified by Unseen-Sequence
      */
-    if ((cp = context_find (usequence))) {
-	char *dp;
+    if ((cp = context_find (usequence)) && *cp) {
+	char **ap, *dp;
 
 	dp = getcpy(cp);
 	ap = brkstring (dp, " ", "\n");
